@@ -439,12 +439,30 @@ XML_Parser XML_ParserCreate(const XML_Char *encodingName)
 
 XML_Parser XML_ParserCreateNS(const XML_Char *encodingName, XML_Char nsSep)
 {
+  static
+  const XML_Char implicitContext[] = {
+    XML_T('x'), XML_T('m'), XML_T('l'), XML_T('='),
+    XML_T('h'), XML_T('t'), XML_T('t'), XML_T('p'), XML_T(':'),
+    XML_T('/'), XML_T('/'), XML_T('w'), XML_T('w'), XML_T('w'),
+    XML_T('.'), XML_T('w'), XML_T('3'),
+    XML_T('.'), XML_T('o'), XML_T('r'), XML_T('g'),
+    XML_T('/'), XML_T('X'), XML_T('M'), XML_T('L'),
+    XML_T('/'), XML_T('1'), XML_T('9'), XML_T('9'), XML_T('8'),
+    XML_T('/'), XML_T('n'), XML_T('a'), XML_T('m'), XML_T('e'),
+    XML_T('s'), XML_T('p'), XML_T('a'), XML_T('c'), XML_T('e'),
+    XML_T('\0')
+  };
+
   XML_Parser parser = XML_ParserCreate(encodingName);
   if (parser) {
     XmlInitEncodingNS(&initEncoding, &encoding, 0);
     ns = 1;
     namespaceSeparator = nsSep;
   }
+  if (!setContext(parser, implicitContext)) {
+    XML_ParserFree(parser);
+    return 0;
+  }
   return parser;
 }
 
@@ -1048,6 +1066,7 @@ doContent(XML_Parser parser,
 	      return XML_ERROR_NO_MEMORY;
 	    if (!externalEntityRefHandler(parser, context, dtd.base, entity->systemId, entity->publicId))
 	      return XML_ERROR_EXTERNAL_ENTITY_HANDLING;
+	    poolDiscard(&tempPool);
 	  }
 	  else if (defaultHandler)
 	    reportDefault(parser, enc, s, next);
@@ -1704,6 +1723,8 @@ processXmlDecl(XML_Parser parser, int isGeneralTextEntity,
 		           &newEncoding,
 		           &standalone))
     return XML_ERROR_SYNTAX;
+  if (!isGeneralTextEntity && standalone == 1)
+    dtd.standalone = 1;
   if (defaultHandler)
     reportDefault(parser, encoding, s, next);
   if (!protocolEncodingName) {
@@ -1730,8 +1751,6 @@ processXmlDecl(XML_Parser parser, int isGeneralTextEntity,
       return result;
     }
   }
-  if (!isGeneralTextEntity && standalone == 1)
-    dtd.standalone = 1;
   return XML_ERROR_NONE;
 }
 
@@ -2730,6 +2749,7 @@ static int dtdInit(DTD *p)
   hashTableInit(&(p->attributeIds));
   hashTableInit(&(p->prefixes));
   p->complete = 1;
+  p->standalone = 0;
   p->base = 0;
   p->defaultPrefix.name = 0;
   p->defaultPrefix.binding = 0;
